/*
 	(c) Copyright 2005 Hewlett-Packard Development Company, LP
 	All rights reserved - see end of file.
 	$Id: ModelAssembler.java,v 1.10 2005/12/07 10:43:04 chris-dollin Exp $
*/

package com.hp.hpl.jena.rdf.model.assemblers;

import java.util.List;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.rdf.model.assemblers.test.TransactionAbortedException;
import com.hp.hpl.jena.shared.*;
import com.hp.hpl.jena.util.IteratorCollection;

public abstract class ModelAssembler extends AssemblerBase implements Assembler
    {
    protected abstract Model createModel( Assembler a, Resource root, Resource type );
    
    public Object create( Assembler a, Resource root, Resource type )
        { 
        Content c = getContent( a, root );
        Model m = createModel( a, root, type );
        if (m.supportsTransactions())
            {
            m.begin();
            try { c.fill( m ); m.commit(); }
            catch (Throwable t) { m.abort(); throw new TransactionAbortedException( t ); }
            }
        else
            c.fill( m );
        m.setNsPrefixes( getPrefixMapping( a, root ) );
        return m; }
    
    private PrefixMapping getPrefixMapping( Assembler a, Resource root )
        {
        return (PrefixMapping) Assembler.prefixMapping.create( a, root, JA.PrefixMapping );
        }

    public Model createModel( Resource resource )
        { return (Model) create( resource ); }

    protected Content getContent( Assembler a, Resource root )
        { 
        List contents = IteratorCollection.iteratorToList( root.listProperties( JA.content ) .mapWith ( new MapObjectToContent( a ) ) );
        return new Content( contents );
        }

    }


/*
 * (c) Copyright 2005 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/