/*
 	(c) Copyright 2005 Hewlett-Packard Development Company, LP
 	All rights reserved - see end of file.
 	$Id: OntModelAssembler.java,v 1.2 2005/12/02 14:21:14 chris-dollin Exp $
*/

package com.hp.hpl.jena.rdf.model.assemblers;

import java.lang.reflect.*;

import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.shared.JenaException;

public class OntModelAssembler extends InfModelAssembler implements Assembler
    {
    public Model createModel( Assembler a, Resource root, Resource type )
        {
        OntModelSpec oms = getOntModelSpec( root );
        Model baseModel = getBase( a, root );
        return ModelFactory.createOntologyModel( oms, baseModel );
        }

    protected OntModelSpec getOntModelSpec( Resource root )
        {
        Resource r = getUniqueResource( root, JA.ontModelSpec );
        if (r != null)
            {
            if (r.hasProperty( null, (RDFNode) null )) 
                {
                throw new JenaException( "haven't done this yet" );
                }
            else
                {
                OntModelSpec oms = getOntModelSpecField( r.getLocalName() );
                if (oms == null) throw new JenaException( "no such OntModelSpec: " + r );
                return oms;                
                }
            }
        return OntModelSpec.OWL_MEM_RDFS_INF;
        }

    protected OntModelSpec getOntModelSpecField( String name )
        {
        try 
            { 
            Class omc = OntModelSpec.class;
            Field f = omc.getField( name ); 
            int mods = f.getModifiers();
            if (f.getType() == omc && isConstant( mods )) 
                return (OntModelSpec) f.get( null );            
            }
        catch (Exception e) 
            {}
        return null;
        }

    protected static boolean isConstant( int mods )
        { return Modifier.isPublic( mods ) && Modifier.isFinal( mods ) && Modifier.isStatic( mods ); }
    }


/*
 * (c) Copyright 2005 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/