/*
 	(c) Copyright 2005 Hewlett-Packard Development Company, LP
 	All rights reserved - see end of file.
 	$Id: ReasonerAssembler.java,v 1.9 2005/12/07 10:43:04 chris-dollin Exp $
*/

package com.hp.hpl.jena.rdf.model.assemblers;

import java.util.*;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.reasoner.*;
import com.hp.hpl.jena.reasoner.rulesys.*;

public class ReasonerAssembler extends AssemblerBase implements Assembler
    {
    public Object create( Assembler a, Resource root, Resource type )
        { 
        Reasoner r = getReasoner( root );
        return addRules( root, a, r );
        }

    private Reasoner addRules( Resource root, Assembler a, Reasoner r )
        {
        List rules = getRules( a, root );
        if (rules.size() > 0l)
            if (r instanceof GenericRuleReasoner)
                {
                ((GenericRuleReasoner) r).addRules( rules );
                }
            else
                throw new CannotHaveRulesException( root );
        return r;
        }

    private List getRules( Assembler a, Resource root )
        {
        List result = new ArrayList();
        StmtIterator it = root.listProperties( JA.rules );
        while (it.hasNext())
            {
            Statement s = it.nextStatement();
            List part = (List) a.create( s.getResource() );
            result.addAll( part );
            }
        return result;
        }

    protected Reasoner getReasoner( Resource root )
        {
        Resource r = getUniqueResource( root, JA.reasonerURL );
        if (r == null)
            return GenericRuleReasonerFactory.theInstance().create( root ); 
        else
            {
            String url = r.getURI();
            ReasonerFactory factory = ReasonerRegistry.theRegistry().getFactory( url );
            if (factory == null) throw new UnknownReasonerException( root, r );
            return factory.create( null );
            }
        }
    }


/*
 * (c) Copyright 2005 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/