/*
 	(c) Copyright 2005 Hewlett-Packard Development Company, LP
 	All rights reserved - see end of file.
 	$Id: AssemblerTestBase.java,v 1.6 2005/12/06 09:34:34 chris-dollin Exp $
*/

package com.hp.hpl.jena.rdf.model.assemblers.test;


import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.rdf.model.assemblers.*;
import com.hp.hpl.jena.rdf.model.test.ModelTestBase;
import com.hp.hpl.jena.shared.*;

public class AssemblerTestBase extends ModelTestBase
    {
    protected static final class ConstantMockAssembler implements Assembler
        {
        private final Object x;
        
        protected ConstantMockAssembler( Object x )
            { this.x = x; }
        
        public Object create( Assembler a, Resource root )
            { return x; }
        
        public Object create( Resource root )
            { return create( this, root ); }
        
        public Model createModel( Resource root )
            { return (Model) create( root ); }

        public Object create( Assembler a, Resource root, Resource type )
            { return x; }
        }

    public AssemblerTestBase( String name )
        { super( name ); }
    
    protected static Model model( String string )
        { 
        Model result = createModel( ReificationStyle.Standard );
        result.setNsPrefix( "ja", JA.getURI() );
        return modelAdd( result, string );
        }

    protected static Resource resourceInModel( String string )
        {
        Model m = model( string );
        Resource r = resource( string.substring( 0, string.indexOf( ' ' ) ) );
        return (Resource) r.inModel( m );        
        }

    protected void testDemandsMinimalType( Assembler a, Resource type )
        {
        try
            { a.createModel( resourceInModel( "x rdf:type rdf:Resource" ) ); 
            fail( "should trap insufficient type" ); }
        catch (MustBeTypedException e)
            { assertEquals( type, e.getType() ); }
        }

    protected void assertSamePrefixMapping( PrefixMapping wanted, PrefixMapping got )
        {
        if (!wanted.samePrefixMappingAs( got ))
            fail( "wanted: " + wanted + " but got: " + got );
        }
    }


/*
 * (c) Copyright 2005 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/