/*
 	(c) Copyright 2005 Hewlett-Packard Development Company, LP
 	All rights reserved - see end of file.
 	$Id: TestContentAssembler.java,v 1.5 2005/12/05 15:27:12 chris-dollin Exp $
*/

package com.hp.hpl.jena.rdf.model.assemblers.test;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.rdf.model.assemblers.*;
import com.hp.hpl.jena.util.FileManager;

public class TestContentAssembler extends AssemblerTestBase
    {
    public TestContentAssembler( String name )
        { super( name ); }

    public void testContent()
        {
        Assembler a = Assembler.content;
        Content c = (Content) a.create( resourceInModel( "x rdf:type ja:Content" ) );
        assertNotNull( c );
        Model m = ModelFactory.createDefaultModel();
        c.fill( m );
        assertEquals( 0, m.size() );
        }
    
    public void testMultipleLiteralsThrowException()
        {
        Resource root = resourceInModel( "x rdf:type ja:LiteralContent; x ja:literalContent 'xxx'; x ja:literalContent 'yyy'" );
        try
            { Assembler.content.create( root ); fail( "should detect multiple literals" ); }
        catch (NotUniqueException e)
            { assertEquals( JA.literalContent, e.getProperty() );  
            assertEquals( root, e.getRoot() ); }
        }
    
    public void testN3StringContentSingleTriples()
        {
        testStringContent( "_x rdf:value '17'xsd:integer", "_:x rdf:value 17 ." );
        testStringContent( "_x rdf:value '42'xsd:integer", "_:x rdf:value 42 ." );
        testStringContent( "_x rdfs:label 'cosmetic'", "_:x rdfs:label 'cosmetic' ." );
        testStringContent( "_x owl:sameAs spoo", "_:x owl:sameAs <eh:/spoo> ." );
        testStringContent( "_x rdf:value '17'xsd:something", "_:x rdf:value '17'^^xsd:something ." );
        testStringContent( "_x dc:title 'A\\sTitle'", "_:x dc:title 'A Title' ." );
        }
    
    public void testN3StringContentMultipleTriples()
        {
        testStringContent( "x rdf:value 5; y owl:sameAs x", "<eh:/x> rdf:value 5 . <eh:/y> owl:sameAs <eh:/x> ." );
        }
    
    public void testSingleExternalContent()
        {
        String source = "kers/testing/assemblers/schema.n3";
        Resource root = resourceInModel( "x rdf:type ja:ExternalContent; x ja:externalContent file:" + source );
        Content c = (Content) Assembler.content.create( root );
        assertIsoModels( FileManager.get().loadModel( "file:" + source ), c.fill( model( "" ) ) );
        }    
    
    public void testMultipleExternalContent()
        {
        String sourceA = "kers/testing/assemblers/schema.n3";
        String sourceB = "kers/testing/assemblers/schema2.n3";
        Resource root = resourceInModel( "x rdf:type ja:ExternalContent; x ja:externalContent file:" + sourceA + "; x ja:externalContent file:" + sourceB );
        Content c = (Content) Assembler.content.create( root );
        Model wanted = FileManager.get().loadModel( "file:" + sourceA ).add( FileManager.get().loadModel( "file:" + sourceB ) );
        assertIsoModels( wanted, c.fill( model( "" ) ) );
        }
    
    public void testMixedContent()
        {
        String source = "kers/testing/assemblers/schema.n3";
        Resource root = resourceInModel
            ( "x rdf:type ja:LiteralContent; x rdf:type ja:ExternalContent" 
            + "; x ja:literalContent '<eh:/eggs>\\srdf:type\\srdf:Property.'"
            + "; x ja:externalContent file:" + source );
        Content c = (Content) Assembler.content.create( root );
        Model wanted = FileManager.get().loadModel( "file:" + source ).add( model( "eggs rdf:type rdf:Property" ) );
        assertIsoModels( wanted, c.fill( model( "" ) ) );
        }
    
    public void testContentLoadsPrefixMappings()
        {
        String content = 
            "@prefix foo: <eh:/foo#>. <eh:/eggs> rdf:type rdf:Property."
            .replaceAll( " ", "\\\\s" );
        Resource root = resourceInModel( "x rdf:type ja:LiteralContent; x ja:literalContent '" + content + "'" );
        Content c = (Content) Assembler.content.create( root );
        Model m = ModelFactory.createDefaultModel();
        c.fill( m );
        assertEquals( "eh:/foo#", m.getNsPrefixURI( "foo" ) );
        }
    
    protected void testStringContent( String mx, String n3 )
        {
        Resource root = resourceInModel( "x rdf:type ja:LiteralContent; x ja:literalContent '" + n3.replaceAll( " ", "\\\\s" ) + "'" );
        Content c = (Content) Assembler.content.create( root );
        Model m = ModelFactory.createDefaultModel();
        c.fill( m );
        assertIsoModels( model( mx ), m );
        }
    }


/*
 * (c) Copyright 2005 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/