/*
 	(c) Copyright 2005 Hewlett-Packard Development Company, LP
 	All rights reserved - see end of file.
 	$Id: TestReasonerAssembler.java,v 1.7 2005/12/01 09:49:09 chris-dollin Exp $
*/

package com.hp.hpl.jena.rdf.model.assemblers.test;

import java.util.*;

import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.assemblers.*;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.dig.*;
import com.hp.hpl.jena.reasoner.rulesys.*;
import com.hp.hpl.jena.reasoner.transitiveReasoner.*;

public class TestReasonerAssembler extends AssemblerTestBase
    {
    public TestReasonerAssembler( String name )
        { super( name ); }

    public void testCreateReasoner()
        {
        Resource root = resourceInModel( "x rdf:type ja:Reasoner" );
        assertInstanceOf( GenericRuleReasoner.class, Assembler.reasoner.create( root ) );
        }
    
    public void testStandardReasonerURLs()
        {
        testReasonerURL( GenericRuleReasoner.class, GenericRuleReasonerFactory.URI );
        testReasonerURL( TransitiveReasoner.class, TransitiveReasonerFactory.URI );
        testReasonerURL( RDFSRuleReasoner.class, RDFSRuleReasonerFactory.URI );
        testReasonerURL( OWLFBRuleReasoner.class, OWLFBRuleReasonerFactory.URI );
        testReasonerURL( DAMLMicroReasoner.class, DAMLMicroReasonerFactory.URI );
        testReasonerURL( DIGReasoner.class, DIGReasonerFactory.URI );
        testReasonerURL( OWLMicroReasoner.class, OWLMicroReasonerFactory.URI );
        testReasonerURL( OWLMiniReasoner.class, OWLMiniReasonerFactory.URI );
        }
    
    public void testBadReasonerURLFails()
        {
        Resource root = resourceInModel( "x rdf:type ja:Reasoner; x ja:reasonerURL bad:URL" );
        try 
            { Assembler.reasoner.create( root ); 
            fail( "should detected unknown reasoner" ); }
        catch (UnknownReasonerException e) 
            { assertEquals( resource( "bad:URL" ), e.getURL() ); }
        }

    public void testMultipleURLsFails()
        {
        Resource root = resourceInModel( "x rdf:type ja:Reasoner; x ja:reasonerURL bad:URL; x ja:reasonerURL another:bad/URL" );
        try 
            { Assembler.reasoner.create( root ); 
            fail( "should detected multiple reasoner URLs" ); }
        catch (NotUniqueException e) 
            { assertEquals( JA.reasonerURL, e.getProperty() ); 
            assertEquals( root, e.getRoot() ); }
        }
    
    public void testOnlyGenericReasonerCanHaveRules()
        {
        String url = TransitiveReasonerFactory.URI;
        Resource root = resourceInModel( "x rdf:type Reasoner; x ja:rules S; x ja:reasonerURL " + url );
        String ruleStringA = "[rdfs2:  (?x ?p ?y), (?p rdfs:domain ?c) -> (?x rdf:type ?c)]";
        final List rules = Rule.parseRules( ruleStringA );
        try 
            { Assembler.reasoner.create( new ConstantMockAssembler( rules ), root );
            fail( "only GenericRuleReasoners can have attached rules" ); }
        catch (CannotHaveRulesException e )
            {
            }
        }
    
    public void testSchema()
        {
        }
    
    public void testSingleRules()
        {
        Resource root = resourceInModel( "x rdf:type ja:Reasoner; x ja:rules S" );
        String ruleStringA = "[rdfs2:  (?x ?p ?y), (?p rdfs:domain ?c) -> (?x rdf:type ?c)]";
        final List rules = Rule.parseRules( ruleStringA );
        Assembler mock = new AssemblerBase() 
            {
            public Object create( Assembler a, Resource root, Resource type )
                {
                assertEquals( root, resource( "S" ) );
                return rules; 
                }
            };
        Reasoner r = (Reasoner) Assembler.reasoner.create( mock, root );
        GenericRuleReasoner grr = (GenericRuleReasoner) r;
        assertEquals( new HashSet( rules ), new HashSet( grr.getRules() ) );
        }
    
    public void testMultipleRules()
        {
        Resource root = resourceInModel( "x rdf:type ja:Reasoner; x ja:rules S; x ja:rules T" );
        String ruleStringA = "[rdfs2:  (?x ?p ?y), (?p rdfs:domain ?c) -> (?x rdf:type ?c)]";
        String ruleStringB = "[rdfs9:  (?x rdfs:subClassOf ?y), (?a rdf:type ?x) -> (?a rdf:type ?y)]";
        final List rulesA = Rule.parseRules( ruleStringA );
        final List rulesB = Rule.parseRules( ruleStringB );
        Assembler mock = new AssemblerBase() 
            {
            public Object create( Assembler a, Resource root, Resource type )
                {
                if (root.equals( resource( "S" ) )) return rulesA;
                if (root.equals( resource( "T" ) )) return rulesB;
                throw new RuntimeException( "unknown resource in mock: " + root );
                }
            };
        Reasoner r = (Reasoner) Assembler.reasoner.create( mock, root );
        GenericRuleReasoner grr = (GenericRuleReasoner) r;
        HashSet wanted = new HashSet();
        wanted.addAll( rulesA );
        wanted.addAll( rulesB );
        assertEquals( wanted, new HashSet( grr.getRules() ) );
        }
    
    protected void testReasonerURL( Class wanted, String string )
        {
        Resource root = resourceInModel( "x rdf:type ja:Reasoner; x ja:reasonerURL " + string );
        assertInstanceOf( wanted, (Reasoner) Assembler.reasoner.create( root ) );
        }
    }


/*
 * (c) Copyright 2005 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/