/*
 	(c) Copyright 2005 Hewlett-Packard Development Company, LP
 	All rights reserved - see end of file.
 	$Id: QuadHashedTripleBunch.java,v 1.1 2005/11/01 15:42:40 chris-dollin Exp $
*/

package hashing;

/*
    (c) Copyright 2005 Hewlett-Packard Development Company, LP
    All rights reserved - see end of file.
    $Id: QuadHashedTripleBunch.java,v 1.1 2005/11/01 15:42:40 chris-dollin Exp $
*/
import java.util.Iterator;

import com.hp.hpl.jena.graph.*;
import com.hp.hpl.jena.graph.query.*;
import com.hp.hpl.jena.mem.*;
import com.hp.hpl.jena.util.iterator.*;

public class QuadHashedTripleBunch extends HashCommon implements TripleBunch
    {
    protected int removed = 0;
    
    protected static final Node Z = Node.createAnon();
    
    public static Triple REMOVED = new Triple( Z, Z, Z );
    
    public QuadHashedTripleBunch( TripleBunch b )
        {
        super( pot( (int) (b.size() / loadFactor) + 10 ) );
    //    System.err.println( ">> CREATING " );
        for (Iterator it = b.iterator(); it.hasNext();) add( (Triple) it.next() );        
        }
    
    protected static int pot( int atLeast )
        {
        int result = 1;
        while (result < atLeast) result *= 2;
    //    System.err.println( ">> pot = " + result );
        return result;
        }
    
    public boolean contains( Triple t )
        { 
        boolean b = quadFindSlot( t ) < 0;
    //    System.err.println( ">> contains( " + t + " ) : " + b );
        return b; 
        }    
    
    protected final int quadFindSlot( Object key )
        {
    //    System.err.println( ">> quadFindSlot (" + key + ")" );
    //    System.err.println( ">>  initial index = " + initialIndexFor( key ) );
        int result = qfs( key );
    //    System.err.println( ">>  result = " + result );
        return result;
        }
    
    protected final int qfs( Object key )
        {
        int index = initialIndexFor( key );
        int where = -1;
        int step = 1;
        while (true)
            {
            Object current = keys[index];
            if (current == null) return where < 0 ? index : where; 
            if (current == REMOVED && where < 0) where = index; 
            if (key.equals( current )) return ~index;
//            System.err.println( ">> collision: probe " + step );
            index -= step;
            step += 1;
            if (index < 0) 
                {
//                System.err.println( ">>  wraparound" );
                index += capacity;
                }
            }
        } 
    
    protected int quadFindSlotBySameValueAs( Triple key )
        {
        int index = initialIndexFor( key );
        int where = -1;
        int step = 1;
        while (true)
            {
            Object current = keys[index];
            if (current == null) return where < 0 ? index : where; 
            if (current == REMOVED && where < 0) where = index; 
            else if (key.matches( (Triple) current )) return ~index;
            index -= step;
            step += 1;
            if (index < 0) index += capacity;
            }
        }
    
    public boolean containsBySameValueAs( Triple t )
        { return quadFindSlotBySameValueAs( t ) < 0; }
    
    public int size()
        { return size; }
    
    public void add( Triple t )
        {
        int where = quadFindSlot( t );
    //    System.err.println( ">> add( " + t + " ) at " + where );
        if (keys[where] == REMOVED) removed -= 1;
        keys[where] = t;
        size += 1;
        if (size + removed > threshold) grow();
        }
    
    protected void grow()
        {
        Object [] oldContents = keys;
        final int oldCapacity = capacity;
        if (size > threshold) growCapacityAndThreshold();
        keys = new Triple[capacity];
        for (int i = 0; i < oldCapacity; i += 1)
            {
            Object t = oldContents[i];
            if (t != null && t != REMOVED) keys[quadFindSlot( t )] = t;
            }
        }
    
    /**
        Work out the capacity and threshold sizes for a new improved bigger
        table (bigger by a factor of two, at present).
    */
    protected void growCapacityAndThreshold()
        {
        capacity = capacity * 2;
        threshold = (int) (capacity * loadFactor);
        }
    
    public void remove( Triple t )
        {
        int where = quadFindSlot( t );
    //    System.err.println( ">> removing ( " + t + " ) from " + ~where );
        keys[~where] = REMOVED;
        removed += 1;
        size -= 1;
        }
    
    public ExtendedIterator iterator()
        {
    //    System.err.println( ";;; -- iterator -------------------------------------------" );
    //    for (int i = 0; i < capacity; i += 1)
    //        {
    //        Object key = keys[i];
    //        System.err.print( i + ": " );
    //        if (key == null) System.err.println( "FREE" );
    //        else if (key == REMOVED) System.err.println( "REMOVED" );
    //        else System.err.println( key );
    //        }
    //    System.err.println( ";;; ++ done ++++++++++++++++++++++++++++++" );
        return new NiceIterator()
            {
            int index = 0;
            int lastIndex = -1;
            Object toRemove = null;
            
            public boolean hasNext()
                {
                while (index < capacity && (keys[index] == null || keys[index] == REMOVED))
                    index += 1;
                return index < capacity;
                }
            
            public Object next()
                {
                if (hasNext() == false) noElements( "" );
                Object answer = keys[index];
                lastIndex = index;
                toRemove = keys[index];
                index += 1;
                return answer;
                }
            
            public void remove()
                {
                if (keys[lastIndex] != toRemove) throw new RuntimeException( "CME" );
                keys[lastIndex] = REMOVED;
                removed += 1;
                }
            };
        }
    
    public void app( Domain d, StageElement next, MatchOrBind s )
        {
        int i = capacity;
        while (i > 0)
            {
            Object t = keys[--i];
            if (t != null  && t != REMOVED && s.matches( (Triple) t )) next.run( d );
            }
        }
    }


/*
 * (c) Copyright 2005 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/